using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.SpaServices.Webpack;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.EntityFrameworkCore;
using TestMakerFreeWebApp.Data;
using Microsoft.Extensions.DependencyInjection;

namespace TestMakerFree
{
   public class Startup
   {
      public Startup(IConfiguration configuration)
      {
         Configuration = configuration;
      }

      public IConfiguration Configuration { get; }

      // This method gets called by the runtime. Use this method to add services to the container.
      public void ConfigureServices(IServiceCollection services)
      {
         services.AddMvc();

         // Dodanie obsugi EntityFramework zwizanej z SqlServer.
         services.AddEntityFrameworkSqlServer();

         // Dodanie ApplicationDbContext.
         services.AddDbContext<ApplicationDbContext>(options =>
            options.UseSqlServer(Configuration.GetConnectionString("DefaultConnection"))
         );
      }

      // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
      public void Configure(IApplicationBuilder app, IHostingEnvironment env)
      {
         if (env.IsDevelopment())
         {
            app.UseDeveloperExceptionPage();
            app.UseWebpackDevMiddleware(new WebpackDevMiddlewareOptions
            {
               HotModuleReplacement = true
            });
         }
         else
         {
            app.UseExceptionHandler("/Home/Error");
         }

         app.UseStaticFiles(new StaticFileOptions()
         {
            OnPrepareResponse = (context) =>
            {
                   // Wyczenie stosowania pamici podrcznej dla wszystkich plikw statycznych. 
                   context.Context.Response.Headers["Cache-Control"] =
                    Configuration["StaticFiles:Headers:Cache-Control"];
               context.Context.Response.Headers["Pragma"] =
                       Configuration["StaticFiles:Headers:Pragma"];
               context.Context.Response.Headers["Expires"] =
                       Configuration["StaticFiles:Headers:Expires"];
            }
         });

         app.UseMvc(routes =>
         {
            routes.MapRoute(
                   name: "default",
                   template: "{controller=Home}/{action=Index}/{id?}");

            routes.MapSpaFallbackRoute(
                   name: "spa-fallback",
                   defaults: new { controller = "Home", action = "Index" });
         });

         // Utwrz zakres usugi, aby otrzyma instancj ApplicationDbContext dziki wstrzykiwaniu zalenoci.
         using (var serviceScope =
         app.ApplicationServices.GetRequiredService<IServiceScopeFactory>
         ().CreateScope())
         {
            var dbContext =
            serviceScope.ServiceProvider.GetService<ApplicationDbContext>();
            // Utwrz baz danych, jeli nie istnieje i zastosuj wszystkie oczekujce migracje.
            dbContext.Database.Migrate();
            // Wypenij baz danymi pocztkowymi.
            DbSeeder.Seed(dbContext);
         }
      }
   }
}
